<?php

namespace Drupal\commerce_cashfree_payment\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "Cashfree_redirect",
 *   label = "Cashfree Redirect Payment",
 *   display_label = "Cashfree Redirect Payment",
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_cashfree_payment\PluginForm\CashfreeRedirect\PaymentCashfreeForm",
 *   },
 * )
 */
class CashfreeRedirect extends OffsitePaymentGatewayBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $supp_currency = [
      'INR' => 'Indian Rupee',
      'USD' => 'United States Dollar',
      'SGD' => 'Singapore Dollar',
      'EUR' => 'Euro official currency of Eurozone',
    ];
    $form['app_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cashfree App ID'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $this->configuration['app_id'],
      '#required' => TRUE,
    ];
    $form['secretkey'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Cashfree Secretkey'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $this->configuration['secretkey'],
      '#required' => TRUE,
    ];
    $form['currency'] = [
      '#type' => 'select',
      '#title' => $this->t('Default Currency'),
      '#options' => $supp_currency,
      '#default_value' => 'INR',
    ];
    $form['pmode'] = [
      '#type' => 'hidden',
      '#title' => $this->t('Mode'),
      '#default_value' => $this->getMode(),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['app_id'] = $values['app_id'];
      $this->configuration['secretkey'] = $values['secretkey'];
      $this->configuration['currency'] = $values['currency'];
      $this->configuration['pmode'] = $values['pmode'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $orderId = $_POST["orderId"];
    $orderAmount = $_POST["orderAmount"];
    $referenceId = $_POST["referenceId"];
    $txStatus = $_POST["txStatus"];
    $paymentMode = $_POST["paymentMode"];
    $txMsg = $_POST["txMsg"];
    $txTime = $_POST["txTime"];
    $signature = $_POST["signature"];
    $merchant_reference = $_POST["referenceId"];
    $data = $orderId . $orderAmount . $referenceId . $txStatus . $paymentMode . $txMsg . $txTime;
    $secretkey = \Drupal::config("commerce_payment.commerce_payment_gateway." . $this->entityId)->getRawData()['configuration']['secretkey'];
    $hash_hmac = hash_hmac('sha256', $data, $secretkey, TRUE);
    $computedSignature = base64_encode($hash_hmac);

    if ($signature == $computedSignature && $txStatus == 'SUCCESS') {
      $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
      $payment = $payment_storage->create([
        'state' => 'SUCCESS',
        'amount' => $order->getTotalPrice(),
        'payment_gateway' => $this->entityId,
        'order_id' => $order->id(),
        'test' => $this->getMode() == 'test',
        'remote_state' => $txStatus,
        'authorized' => $this->time->getRequestTime(),
        'remote_id' => $merchant_reference,
      ]);
      $payment->save();
      $this->messenger()->addStatus($this->t('Your payment was successful'));
    }
    else {
      $this->messenger()->addError($this->t('Your payment was canceled'));
    }
  }

}
